@@ -41,11 +41,17 @@ KDE_NO_EXPORT void ListsSource::play (KMPlayer::Mrl *mrl) {
 }
 
 KDE_NO_EXPORT void ListsSource::activate () {
+    activated = true;
     play (m_current ? m_current->mrl () : NULL);
 }
 
+QString ListsSource::prettyName ()
+{
+    return ((KMPlayer::PlaylistRole *)m_document->role (KMPlayer::RolePlaylist))->caption ();
+}
+
 KDE_NO_CDTOR_EXPORT FileDocument::FileDocument (short i, const QString &s, KMPlayer::Source *src)
- : KMPlayer::SourceDocument (src, s) {
+ : KMPlayer::SourceDocument (src, s), load_tree_version ((unsigned int)-1) {
     id = i;
 }
 
@@ -58,20 +64,27 @@ KDE_NO_EXPORT KMPlayer::Node *FileDocument::childFromTag(const QString &tag) {
 void FileDocument::readFromFile (const QString & fn) {
     QFile file (fn);
     kDebug () << "readFromFile " << fn;
-    if (file.exists ()) {
-        file.open (IO_ReadOnly);
+    if (QFileInfo (file).exists ()) {
+        file.open (QIODevice::ReadOnly);
         QTextStream inxml (&file);
         KMPlayer::readXML (this, inxml, QString (), false);
         normalize ();
     }
+    load_tree_version = m_tree_version;
 }
 
 void FileDocument::writeToFile (const QString & fn) {
     QFile file (fn);
     kDebug () << "writeToFile " << fn;
-    file.open (IO_WriteOnly);
-    QByteArray utf = outerXML ().toUtf8 ();
-    file.writeBlock (utf, utf.size ());
+    file.open (QIODevice::WriteOnly | QIODevice::Truncate);
+    file.write (outerXML ().toUtf8 ());
+    load_tree_version = m_tree_version;
+}
+
+void FileDocument::sync (const QString &fn)
+{
+    if (resolved && load_tree_version != m_tree_version)
+        writeToFile (fn);
 }
 
 KDE_NO_CDTOR_EXPORT Recents::Recents (KMPlayerApp *a)
@@ -102,7 +115,7 @@ KDE_NO_EXPORT KMPlayer::Node *Recents::childFromTag (const QString & tag) {
 }
 
 KDE_NO_EXPORT void Recents::message (KMPlayer::MessageType msg, void *data) {
-    if (KMPlayer::MsgChildFinished)
+    if (KMPlayer::MsgChildFinished == msg)
         finish ();
     else
         FileDocument::message (msg, data);
@@ -112,12 +125,12 @@ KDE_NO_CDTOR_EXPORT
 Recent::Recent (KMPlayer::NodePtr & doc, KMPlayerApp * a, const QString &url)
   : KMPlayer::Mrl (doc, id_node_recent_node), app (a) {
     src = url;
-    setAttribute (KMPlayer::StringPool::attr_url, url);
+    setAttribute (KMPlayer::Ids::attr_url, url);
 }
 
 KDE_NO_EXPORT void Recent::closed () {
-    if (src.isEmpty ())
-        src = getAttribute (KMPlayer::StringPool::attr_url);
+    src = getAttribute (KMPlayer::Ids::attr_url);
+    Mrl::closed ();
 }
 
 KDE_NO_EXPORT void Recent::activate () {
@@ -126,10 +139,10 @@ KDE_NO_EXPORT void Recent::activate () {
 
 KDE_NO_CDTOR_EXPORT
 Group::Group (KMPlayer::NodePtr & doc, KMPlayerApp * a, const QString & pn)
-  : KMPlayer::Title (doc, KMPlayer::id_node_group_node), app (a) {
+  : KMPlayer::Element (doc, KMPlayer::id_node_group_node), app (a) {
     title = pn;
     if (!pn.isEmpty ())
-        setAttribute (KMPlayer::StringPool::attr_title, pn);
+        setAttribute (KMPlayer::Ids::attr_title, pn);
 }
 
 KDE_NO_EXPORT KMPlayer::Node *Group::childFromTag (const QString & tag) {
@@ -141,8 +154,15 @@ KDE_NO_EXPORT KMPlayer::Node *Group::childFromTag (const QString & tag) {
 }
 
 KDE_NO_EXPORT void Group::closed () {
-    if (title.isEmpty ())
-        title = getAttribute (KMPlayer::StringPool::attr_title);
+    title = getAttribute (KMPlayer::Ids::attr_title);
+    Element::closed ();
+}
+
+void *Group::role (KMPlayer::RoleType msg, void *content)
+{
+    if (KMPlayer::RolePlaylist == msg)
+        return (KMPlayer::PlaylistRole *) this ;
+    return Element::role (msg, content);
 }
 
 KDE_NO_EXPORT void Playlist::defer () {
@@ -170,7 +190,8 @@ KDE_NO_CDTOR_EXPORT Playlist::Playlist (KMPlayerApp *a, KMPlayer::Source *s, boo
 
 KDE_NO_EXPORT KMPlayer::Node *Playlist::childFromTag (const QString & tag) {
     // kDebug () << nodeName () << " childFromTag " << tag;
-    const char * name = tag.ascii ();
+    QByteArray ba = tag.toUtf8 ();
+    const char *name = ba.constData ();
     if (!strcmp (name, "item"))
         return new PlaylistItem (m_doc, app, playmode);
     else if (!strcmp (name, "group"))
@@ -181,7 +202,7 @@ KDE_NO_EXPORT KMPlayer::Node *Playlist::childFromTag (const QString & tag) {
 }
 
 KDE_NO_EXPORT void Playlist::message (KMPlayer::MessageType msg, void *data) {
-    if (KMPlayer::MsgChildFinished && !playmode)
+    if (KMPlayer::MsgChildFinished == msg && !playmode)
         finish ();
     else
         FileDocument::message (msg, data);
@@ -190,6 +211,7 @@ KDE_NO_EXPORT void Playlist::message (KMPlayer::MessageType msg, void *data) {
 KDE_NO_CDTOR_EXPORT
 PlaylistItemBase::PlaylistItemBase (KMPlayer::NodePtr &d, short i, KMPlayerApp *a, bool pm)
     : KMPlayer::Mrl (d, i), app (a), playmode (pm) {
+    editable = !pm;
 }
 
 KDE_NO_EXPORT void PlaylistItemBase::activate () {
@@ -206,7 +228,7 @@ KDE_NO_EXPORT void PlaylistItemBase::activate () {
             data = QString ("<playlist>") +
                 parentNode ()->innerXML () +
                 QString ("</playlist>");
-            pn = parentNode ()->caption ();
+            pn = ((KMPlayer::PlaylistRole *)parentNode ()->role (KMPlayer::RolePlaylist))->caption ();
         } else {
             data = outerXML ();
             pn = title.isEmpty () ? src : title;
@@ -238,20 +260,19 @@ KDE_NO_EXPORT void PlaylistItemBase::activate () {
 }
 
 void PlaylistItemBase::closed () {
-    if (title.isEmpty ())
-        title = getAttribute (KMPlayer::StringPool::attr_title);
+    title = getAttribute (KMPlayer::Ids::attr_title);
+    Mrl::closed ();
 }
 
 KDE_NO_CDTOR_EXPORT
 PlaylistItem::PlaylistItem (KMPlayer::NodePtr & doc, KMPlayerApp *a, bool pm, const QString &url)
  : PlaylistItemBase (doc, KMPlayer::id_node_playlist_item, a, pm) {
     src = url;
-    setAttribute (KMPlayer::StringPool::attr_url, url);
+    setAttribute (KMPlayer::Ids::attr_url, url);
 }
 
 KDE_NO_EXPORT void PlaylistItem::closed () {
-    if (src.isEmpty ())
-        src = getAttribute (KMPlayer::StringPool::attr_url);
+    src = getAttribute (KMPlayer::Ids::attr_url);
     PlaylistItemBase::closed ();
 }
 
@@ -263,25 +284,39 @@ KDE_NO_EXPORT void PlaylistItem::begin () {
 }
 
 KDE_NO_EXPORT void PlaylistItem::setNodeName (const QString & s) {
-    src = s;
-    setAttribute (KMPlayer::StringPool::attr_url, s);
+    bool uri = s.startsWith (QChar ('/'));
+    if (!uri) {
+        int p = s.indexOf ("://");
+        uri = p > 0 && p < 10;
+    }
+    if (uri) {
+        if (title.isEmpty () || title == src)
+            title = s;
+        src = s;
+        setAttribute (KMPlayer::Ids::attr_url, s);
+    } else {
+        title = s;
+        setAttribute (KMPlayer::Ids::attr_title, s);
+    }
 }
 
 KDE_NO_CDTOR_EXPORT
 PlaylistGroup::PlaylistGroup (KMPlayer::NodePtr &doc, KMPlayerApp *a, const QString &pn)
-  : KMPlayer::Title (doc, KMPlayer::id_node_group_node), app (a), playmode (false) {
+  : KMPlayer::Element (doc, KMPlayer::id_node_group_node), app (a), playmode (false) {
     title = pn;
     if (!pn.isEmpty ())
-        setAttribute (KMPlayer::StringPool::attr_title, pn);
+        setAttribute (KMPlayer::Ids::attr_title, pn);
 }
 
 KDE_NO_CDTOR_EXPORT
 PlaylistGroup::PlaylistGroup (KMPlayer::NodePtr &doc, KMPlayerApp *a, bool lm)
-  : KMPlayer::Title (doc, KMPlayer::id_node_group_node), app (a), playmode (lm) {
+  : KMPlayer::Element (doc, KMPlayer::id_node_group_node), app (a), playmode (lm) {
+    editable = !lm;
 }
 
 KDE_NO_EXPORT KMPlayer::Node *PlaylistGroup::childFromTag (const QString &tag) {
-    const char * name = tag.ascii ();
+    QByteArray ba = tag.toUtf8 ();
+    const char *name = ba.constData ();
     if (!strcmp (name, "item"))
         return new PlaylistItem (m_doc, app, playmode);
     else if (!strcmp (name, "group"))
@@ -292,13 +327,20 @@ KDE_NO_EXPORT KMPlayer::Node *PlaylistGroup::childFromTag (const QString &tag) {
 }
 
 KDE_NO_EXPORT void PlaylistGroup::closed () {
-    if (title.isEmpty ())
-        title = getAttribute (KMPlayer::StringPool::attr_title);
+    title = getAttribute (KMPlayer::Ids::attr_title);
+    Element::closed ();
 }
 
 KDE_NO_EXPORT void PlaylistGroup::setNodeName (const QString &t) {
     title = t;
-    setAttribute (KMPlayer::StringPool::attr_title, t);
+    setAttribute (KMPlayer::Ids::attr_title, t);
+}
+
+void *PlaylistGroup::role (KMPlayer::RoleType msg, void *content)
+{
+    if (KMPlayer::RolePlaylist == msg)
+        return (KMPlayer::PlaylistRole *) this ;
+    return Element::role (msg, content);
 }
 
 KDE_NO_CDTOR_EXPORT
@@ -316,17 +358,17 @@ KDE_NO_EXPORT void HtmlObject::closed () {
     for (Node *n = firstChild (); n; n = n->nextSibling ()) {
         if (n->id == KMPlayer::id_node_param) {
             KMPlayer::Element *e = static_cast <KMPlayer::Element *> (n);
-            QString name = e->getAttribute (KMPlayer::StringPool::attr_name);
+            QString name = e->getAttribute (KMPlayer::Ids::attr_name);
             if (name == "type")
-                mimetype = e->getAttribute (KMPlayer::StringPool::attr_value);
+                mimetype = e->getAttribute (KMPlayer::Ids::attr_value);
             else if (name == "movie")
-                src = e->getAttribute (KMPlayer::StringPool::attr_value);
+                src = e->getAttribute (KMPlayer::Ids::attr_value);
         } else if (n->id == KMPlayer::id_node_html_embed) {
             KMPlayer::Element *e = static_cast <KMPlayer::Element *> (n);
-            QString type = e->getAttribute (KMPlayer::StringPool::attr_type);
+            QString type = e->getAttribute (KMPlayer::Ids::attr_type);
             if (!type.isEmpty ())
                 mimetype = type;
-            QString asrc = e->getAttribute (KMPlayer::StringPool::attr_src);
+            QString asrc = e->getAttribute (KMPlayer::Ids::attr_src);
             if (!asrc.isEmpty ())
                 src = asrc;
         }
@@ -335,7 +377,8 @@ KDE_NO_EXPORT void HtmlObject::closed () {
 }
 
 KDE_NO_EXPORT KMPlayer::Node *HtmlObject::childFromTag (const QString & tag) {
-    const char *name = tag.ascii ();
+    QByteArray ba = tag.toUtf8 ();
+    const char *name = ba.constData ();
     if (!strcasecmp (name, "param"))
         return new KMPlayer::DarkNode (m_doc, name, KMPlayer::id_node_param);
     else if (!strcasecmp (name, "embed"))
@@ -361,7 +404,7 @@ QString Generator::genReadAsk (KMPlayer::Node *n) {
     QString title;
     QString desc;
     QString type = static_cast <Element *> (n)->getAttribute (
-            KMPlayer::StringPool::attr_type);
+            KMPlayer::Ids::attr_type);
     QString key = static_cast<Element*>(n)->getAttribute ("key");
     QString def = static_cast<Element*>(n)->getAttribute ("default");
     QString input;
@@ -378,10 +421,10 @@ QString Generator::genReadAsk (KMPlayer::Node *n) {
         for (KMPlayer::Node *c = n->firstChild (); c; c = c->nextSibling ())
             switch (c->id) {
                 case id_node_gen_title:
-                    title = c->innerText ().simplifyWhiteSpace ();
+                    title = c->innerText ().simplified ();
                     break;
                 case id_node_gen_description:
-                    desc = c->innerText ().simplifyWhiteSpace ();
+                    desc = c->innerText ().simplified ();
                     break;
             }
         input = KInputDialog::getText (title, desc, def);
@@ -441,7 +484,7 @@ QString Generator::genReadString (KMPlayer::Node *n) {
             str += genReadString (c);
             break;
         case id_node_gen_literal:
-            str += c->innerText ().simplifyWhiteSpace ();
+            str += c->innerText ().simplified ();
             break;
         case id_node_gen_predefined: {
             QString val = static_cast <Element *>(c)->getAttribute ("key");
@@ -458,7 +501,7 @@ QString Generator::genReadString (KMPlayer::Node *n) {
             str += genReadAsk (c);
             break;
         case KMPlayer::id_node_text:
-             str += c->nodeValue ().simplifyWhiteSpace ();
+             str += c->nodeValue ().simplified ();
         }
     if (find_resource)
         str = KStandardDirs().findResource ("data", str);
@@ -500,7 +543,7 @@ void Generator::activate () {
     KMPlayer::Node *n = firstChild ();
     if (n && n->id == id_node_gen_generator) {
         title = static_cast<Element *>(n)->getAttribute (
-                KMPlayer::StringPool::attr_name);
+                KMPlayer::Ids::attr_name);
         for (KMPlayer::Node *c = n->firstChild (); c && !canceled; c = c->nextSibling ())
             switch (c->id) {
             case id_node_gen_input:
@@ -608,7 +651,7 @@ void Generator::readyRead () {
             m_source->setDocument (pl, pl);
             if (reset_only) {
                 m_source->activate ();
-                app->setCaption (getAttribute(KMPlayer::StringPool::attr_name));
+                app->setCaption (getAttribute(KMPlayer::Ids::attr_name));
             } else {
                 app->player ()->setSource (m_source);
             }
